@@ -1,11 +1,11 @@
 /*
  * SPDX-License-Identifier: GPL-3.0-only
- * MuseScore-CLA-applies
+ * MuseScore-Studio-CLA-applies
  *
- * MuseScore
+ * MuseScore Studio
  * Music Composition & Notation
  *
- * Copyright (C) 2021 MuseScore BVBA and others
+ * Copyright (C) 2021 MuseScore Limited
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -23,13 +23,20 @@
 
 using namespace mu::instrumentsscene;
 using namespace mu::notation;
+using namespace muse;
 
-mu::RetVal<PartInstrumentListScoreOrder> SelectInstrumentsScenarioStub::selectInstruments() const
+async::Promise<PartInstrumentListScoreOrder> SelectInstrumentsScenarioStub::selectInstruments() const
 {
-    return make_ret(Ret::Code::NotSupported);
+    return async::make_promise<PartInstrumentListScoreOrder>([](auto, auto reject) {
+        Ret ret = make_ret(Ret::Code::NotSupported);
+        return reject(ret.code(), ret.text());
+    });
 }
 
-mu::RetVal<Instrument> SelectInstrumentsScenarioStub::selectInstrument(const notation::InstrumentKey&) const
+async::Promise<InstrumentTemplate> SelectInstrumentsScenarioStub::selectInstrument(const notation::InstrumentKey&) const
 {
-    return make_ret(Ret::Code::NotSupported);
+    return async::make_promise<InstrumentTemplate>([](auto, auto reject) {
+        Ret ret = make_ret(Ret::Code::NotSupported);
+        return reject(ret.code(), ret.text());
+    });
 }
